#ifndef _CONFIG_H
#define _CONFIG_H

////////////////////////////////////////////////////////////////////////
// Begin IO Pin Assignments

// Stepper Driver Pins
#define E_DIRECTION 0
#define E_STEP 1
#define E_DISABLE 2
#define X_DIRECTION 3
#define X_STEP 4
#define X_DISABLE 5
#define Y_DISABLE 6
#define Y_STEP 7  
#define Y_DIRECTION 8
#define Z_DISABLE 9
#define Z_STEP 10
#define Z_DIRECTION 11

// SD Card Pins - These match the PBOE
#define DO 22
#define CLK 23
#define DI 24
#define CS 25

// I2C Pins
#define SCL 28
#define SDA 29

// Full Duplex Serial Pins
#define TX 30
#define RX 31

// End IO Pin Assignments
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// Begin Full Duplex Serial Constants

// Set the mode for full duplex serial
//  mode 0 = invert rx
//  mode 1 = invert tx
//  mode 2 = open-drain/source tx
//  mode 3 = ignore tx echo on rx
#define GCODE_MODE 0

// Set the baudrate for full duplex serial, eg. 9600, 57600, 115200, etc...
#define GCODE_BAUD 115200

// End Full Duplex Serial Constants
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// Begin I2C Constants

// Set the mode for I2C
//  mode 0 = pulls the line high with pull-up resistor
//  mode 1 = drives the line high with setting I/O pin to output.
#define I2C_MODE 0

// End I2C Constants
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// Begin Stepper Driver Constants

// E Axis Constants
#define E_PULSE_WIDTH (CLKFREQ / 209500) // 381
#define E_MIN_SPEED (CLKFREQ / 5714) // 14000 Rounded
#define E_MAX_SPEED (CLKFREQ / 16000) // 5000
#define E_RAMP_INC_DEC (CLKFREQ / 26666666) // 3 Rounded
#define E_INC_DIR 0 // Counter-clockwise rotation
#define E_DEC_DIR 1 // Clockwise rotation
#define E_STEPS_PER_REV 2000 // 200 Full steps * 10 microsteps
#define E_RAMPING_STEPS ((E_MIN_SPEED - E_MAX_SPEED) / E_RAMP_INC_DEC)
#define E_TOTAL_RAMPING_STEPS (E_RAMPING_STEPS * 2)
#define E_MAX_STEPS 16933 // 12 inches of linear travel (304.8 millimeters)
#define E_STEPS_PER_MM 55.555 // Rounded

// X Axis Constants
#define X_PULSE_WIDTH (CLKFREQ / 209500) // 381
#define X_MIN_SPEED (CLKFREQ / 5714) // 14000 Rounded
#define X_MAX_SPEED (CLKFREQ / 16000) // 5000
#define X_RAMP_INC_DEC (CLKFREQ / 26666666) // 3 Rounded
#define X_INC_DIR 0 // Counter-clockwise rotation
#define X_DEC_DIR 1 // Clockwise rotation
#define X_STEPS_PER_REV 2000 // 200 Full steps * 10 microsteps
#define X_RAMPING_STEPS ((X_MIN_SPEED - X_MAX_SPEED) / X_RAMP_INC_DEC)
#define X_TOTAL_RAMPING_STEPS (X_RAMPING_STEPS * 2)
#define X_MAX_STEPS 16933 // 12 inches of linear travel (304.8 millimeters)
#define X_STEPS_PER_MM 55.555 // Rounded
  
// Y Axis Constants
#define Y_PULSE_WIDTH (CLKFREQ / 209500) // 381
#define Y_MIN_SPEED (CLKFREQ / 5714) // 14000 Rounded
#define Y_MAX_SPEED (CLKFREQ / 16000) // 5000
#define Y_RAMP_INC_DEC (CLKFREQ / 26666666) // 3 Rounded
#define Y_INC_DIR 0 // Counter-clockwise rotation
#define Y_DEC_DIR 1 // Clockwise rotation
#define Y_STEPS_PER_REV 2000 // 200 Full steps * 10 microsteps
#define Y_RAMPING_STEPS ((Y_MIN_SPEED - Y_MAX_SPEED) / Y_RAMP_INC_DEC)
#define Y_TOTAL_RAMPING_STEPS (Y_RAMPING_STEPS * 2)
#define Y_MAX_STEPS 16933 // 12 inches of linear travel (304.8 millimeters)
#define Y_STEPS_PER_MM 55.555

// Z Axis Constants
#define Z_PULSE_WIDTH (CLKFREQ / 209500) // 381
#define Z_MIN_SPEED (CLKFREQ / 5714) // 14000 Rounded
#define Z_MAX_SPEED (CLKFREQ / 16000) // 5000
#define Z_RAMP_INC_DEC (CLKFREQ / 26666666) // 3 Rounded
#define Z_INC_DIR 0 // Counter-clockwise rotation
#define Z_DEC_DIR 1 // Clockwise rotation
#define Z_STEPS_PER_REV 2000 // 200 Full steps * 10 microsteps
#define Z_RAMPING_STEPS ((Z_MIN_SPEED - Z_MAX_SPEED) / Z_RAMP_INC_DEC)
#define Z_TOTAL_RAMPING_STEPS (Z_RAMPING_STEPS * 2)
#define Z_MAX_STEPS 16933 // 12 inches of linear travel (304.8 millimeters)
#define Z_STEPS_PER_MM 55.555 // Rounded

// End Stepper Driver Constants
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// Begin Miscellaneous Constants

#define DEBUG
#define GCODE_ELEMENTS 10

// End  Miscellaneous Constants
////////////////////////////////////////////////////////////////////////

#endif /* _CONFIG_H */