#include "follower_2.h"
#include "config.h"
#include "move_struct.h"
#include "Gearing.h"

static int follower_2_cog = 0;
static int follower_2_stack[100];

volatile bool follower_2_move_complete;

volatile uint8_t follower_2_dir;
volatile uint8_t follower_2_dir_pin;
volatile uint8_t follower_2_step_pin;
volatile uint32_t follower_2_ramp_up_steps;
volatile uint32_t follower_2_running_steps;
volatile uint32_t follower_2_ramp_down_steps;
volatile uint32_t follower_2_pulse_width;
volatile int32_t follower_2_offset;
volatile uint32_t follower_2_start_stop_speed;
volatile uint32_t follower_2_ramp_inc_dec;
volatile double follower_2_int_part;
volatile double follower_2_dec_part;

volatile uint8_t copy_2_masters_pulse_width;

////////////////////////////////////////////////////////////
// This variable is only for testing purposes and it can be
// deleted, after testing is completed.
volatile uint32_t follower_2_steps_made;
////////////////////////////////////////////////////////////

void follower_2(void *par);

void follower_2_init()
{
	if(follower_2_cog == 0)
	{
		follower_2_cog = 1 + cogstart(&follower_2, NULL,
			follower_2_stack, sizeof follower_2_stack);
	}
}

void follower_2_stop()
{
	if(follower_2_cog > 0)
	{
		cogstop(follower_2_cog - 1);
		follower_2_cog = 0;
	}
}

void follower_2(void *par)
{
	do
	{
		if(follower_2_move_now == true)
		{
			follower_2_move_now = false;

			int32_t follower_2_counter;
			int32_t follower_2_step_counter;

			////////////////////////////////////////////////////////////
			// This variable is only for testing purposes and it can be
			// deleted, after testing is completed.
			follower_2_steps_made = 0;
			////////////////////////////////////////////////////////////

			// Set DIRA as an output.
			DIRA |= 1 << follower_2_dir_pin;

			// Set up the CTRMODE of Counter A for NCO/PWM single-ended.
//			CTRA = (4 << 26) | follower_2_step_pin;

			// Set the value to be added to PHSA with every clock cycle.
//			FRQA = 1;

			// Set DIRA as an output.
			DIRA |= 1 << follower_2_step_pin;

			// Set the OUTA register to match the desired direction of rotation.
			if(follower_2_dir == 0)
			{
				OUTA &= (~1 << follower_2_dir_pin);
			}
			else
			{
				OUTA |= 1 << follower_2_dir_pin;
			}

			// Get the current System Counter value.
			follower_2_counter = CNT;

			// Maintain a ramp up consistent with the master's ramp up.
			while(follower_2_ramp_up_steps != 0)
			{
				// Pause for a high pulse on the master's step pin.
				waitcnt(copy_2_masters_pulse_width + CNT);

				// Maintain a step count consistent with the master's steps.
				follower_2_step_counter++;

				// Determine if the follower should make a step, if so, then make it.
				// However keep in mind that all timing is critical at this point.
				// So adjust timers appropriately
				if(follower_2_step_counter % (uint32_t)follower_2_int_part == 0)
				{
					follower_2_offset = (double)follower_2_start_stop_speed * follower_2_dec_part;

					// At this location, place the waitcnt for the time frame before
					// this follower is supposed to make a step.
					waitcnt(follower_2_offset + CNT);

					// At this location, drive the follower's step pin high for the
					// duration of this follower's pulse width (follower_2_pulse_width).
					OUTA |= 1 << follower_2_step_pin;
					waitcnt(follower_2_pulse_width + CNT);

					// At this location, drive the follower's step pin low.
					OUTA &= (~1 << follower_2_step_pin);

					////////////////////////////////////////////////////////////
					// This variable is only for testing purposes and it can be
					// deleted, after testing is completed.
					follower_2_steps_made++;
					////////////////////////////////////////////////////////////

					// The following timimg code needs adjustment to compensate for the
					// if/else conditional statement, computations, the followers waitcnt
					// offset, the drive high instruction, and the drive low instruction.
					// The ultimate goal is to match the same time frame as utilized
					// by the master's waitcnt.
					waitcnt(follower_2_counter += follower_2_start_stop_speed -=
						follower_2_ramp_inc_dec); // 14016 ticks

					// Decrement ramp_up_steps
					follower_2_ramp_up_steps--;
				}
				else
				{
					// The following timimg code needs adjustment to compensate for the
					// if/else conditional statement, so that it will match the master's
					// similar  waitcnt.
					waitcnt(follower_2_counter += follower_2_start_stop_speed -=
						follower_2_ramp_inc_dec -= 266); // 14016 ticks

					// Decrement ramp_up_steps
					follower_2_ramp_up_steps--;
				}
			}

			// Run this driver at speed consistent with the master's speed
			while(follower_2_running_steps != 0)
			{
				// Pause for a high pulse on the master's step pin.
				waitcnt(copy_2_masters_pulse_width + CNT);

				// Maintain a step count consistent with the master's steps.
				follower_2_step_counter++;

				// Determine if the follower should make a step, if so, then make it.
				// However keep in mind that all timing is critical at this point.
				// So adjust timers appropriately
				if(follower_2_step_counter % (uint32_t)follower_2_int_part == 0)
				{
					follower_2_offset = (double)follower_2_start_stop_speed * follower_2_dec_part;

					// At this location, place the waitcnt for the time frame before
					// this follower is supposed to make a step.
					waitcnt(follower_2_offset + CNT);

					// At this location, drive the follower's step pin high for the
					// duration of this follower's pulse width (follower_2_pulse_width).
					OUTA |= 1 << follower_2_step_pin;
					waitcnt(follower_2_pulse_width + CNT);

					// At this location, drive the follower's step pin low.
					OUTA &= (~1 << follower_2_step_pin);

					////////////////////////////////////////////////////////////
					// This variable is only for testing purposes and it can be
					// deleted, after testing is completed.
					follower_2_steps_made++;
					////////////////////////////////////////////////////////////

					// The following timimg code needs adjustment to compensate for the
					// if/else conditional statement, computations, the followers waitcnt
					// offset, the drive high instruction, and the drive low instruction.
					// The ultimate goal is to match the same time frame as utilized
					// by the master's waitcnt.
					waitcnt(follower_2_counter += follower_2_start_stop_speed); // 14016 ticks

					// Decrement running_steps
					follower_2_running_steps--;
				}
				else
				{
					// The following timimg code needs adjustment to compensate for the
					// if/else conditional statement, so that it will match the master's
					// similar  waitcnt.
					waitcnt(follower_2_counter += follower_2_start_stop_speed -= 266); // 14016 ticks

					// Decrement running_steps
					follower_2_running_steps--;
				}
			}

			// Maintain a ramp down consistent with the master's ramp up.
			while(follower_2_ramp_down_steps != 0)
			{
				// Pause for a high pulse on the master's step pin.
				waitcnt(copy_2_masters_pulse_width + CNT);

				// Maintain a step count consistent with the master's steps.
				follower_2_step_counter++;

				// Determine if the follower should make a step, if so, then make it.
				// However keep in mind that all timing is critical at this point.
				// So adjust timers appropriately
				if(follower_2_step_counter % (uint32_t)follower_2_int_part == 0)
				{
					follower_2_offset = (double)follower_2_start_stop_speed * follower_2_dec_part;

					// At this location, place the waitcnt for the time frame before
					// this follower is supposed to make a step.
					waitcnt(follower_2_offset + CNT);

					// At this location, drive the follower's step pin high for the
					// duration of this follower's pulse width (follower_2_pulse_width).
					OUTA |= 1 << follower_2_step_pin;
					waitcnt(follower_2_pulse_width + CNT);

					// At this location, drive the follower's step pin low.
					OUTA &= (~1 << follower_2_step_pin);

					////////////////////////////////////////////////////////////
					// This variable is only for testing purposes and it can be
					// deleted, after testing is completed.
					follower_2_steps_made++;
					////////////////////////////////////////////////////////////

					// The following timimg code needs adjustment to compensate for the
					// if/else conditional statement, computations, the followers waitcnt
					// offset, the drive high instruction, and the drive low instruction.
					// The ultimate goal is to match the same time frame as utilized
					// by the master's waitcnt.
					waitcnt(follower_2_counter += follower_2_start_stop_speed +=
						follower_2_ramp_inc_dec); // 14016 ticks

					// Decrement ramp_down_steps
					follower_2_ramp_down_steps--;
				}
				else
				{
					// The following timimg code needs adjustment to compensate for the
					// if/else conditional statement, so that it will match the master's
					// similar  waitcnt.
					waitcnt(follower_2_counter += follower_2_start_stop_speed +=
						follower_2_ramp_inc_dec -= 266); // 14016 ticks

					// Decrement ramp_down_steps
					follower_2_ramp_down_steps--;
				}
			}

			follower_2_move_complete = true;
		}
	}
	while(follower_2_cog > 0);
}