#include "master.h"
#include "config.h"
#include "move_struct.h"
#include "Gearing.h"

static int master_cog = 0;
static int master_stack[50];

volatile bool master_move_complete;

volatile uint8_t master_dir;
volatile uint8_t master_dir_pin;
volatile uint8_t master_step_pin;
volatile uint32_t master_ramp_up_steps;
volatile uint32_t master_running_steps;
volatile uint32_t master_ramp_down_steps;
volatile uint32_t master_pulse_width;
volatile uint32_t master_start_stop_speed;
volatile uint32_t master_ramp_inc_dec;

////////////////////////////////////////////////////////////
// This variable is only for testing purposes and it can be
// deleted, after testing is completed.
volatile uint32_t master_steps_made;
////////////////////////////////////////////////////////////

void master(void *par);

void master_init()
{
	if(master_cog == 0)
	{
		master_cog = 1 + cogstart(&master, NULL,
			master_stack, sizeof master_stack);
	}
}

void master_stop()
{
	if(master_cog > 0)
	{
		cogstop(master_cog - 1);
		master_cog = 0;
	}
}

void master(void *par)
{
	do
	{
		if(master_move_now == true)
		{
			master_move_now = false;

			int32_t master_counter;
			int32_t master_step_counter;

			////////////////////////////////////////////////////////////
			// This variable is only for testing purposes and it can be
			// deleted, after testing is completed.
			master_steps_made = 0;
			////////////////////////////////////////////////////////////

			// Set DIRA as an output.
			DIRA |= 1 << master_dir_pin;

			// Set up the CTRMODE of Counter A for NCO/PWM single-ended.
			CTRA = (4 << 26) | master_step_pin;

			// Set the value to be added to PHSA with every clock cycle.
			FRQA = 1;

			// Set DIRA as an output.
			DIRA |= 1 << master_step_pin;

			// Set the OUTA register to match the desired direction of rotation.
			if(master_dir == 0)
			{
				OUTA &= (~1 << master_dir_pin);
			}
			else
			{
				OUTA |= 1 << master_dir_pin;
			}

			// Get the current System Counter value.
			master_counter = CNT;

			// Ramp up the stepper motor a predetermined amount of steps
			while(master_ramp_up_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -master_pulse_width;
				master_step_counter++;

				////////////////////////////////////////////////////////////
				// This variable is only for testing purposes and it can be
				// deleted, after testing is completed.
				master_steps_made++;
				////////////////////////////////////////////////////////////

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(master_counter += master_start_stop_speed
					-= master_ramp_inc_dec -= 0);

				// Decrement ramp_up_steps
				master_ramp_up_steps--;
			}

			// Run the stepper motor at the current speed for a predetermined 
			// number of steps
			while(master_running_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -master_pulse_width;
				master_step_counter++;

				////////////////////////////////////////////////////////////
				// This variable is only for testing purposes and it can be
				// deleted, after testing is completed.
				master_steps_made++;
				////////////////////////////////////////////////////////////

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(master_counter += master_start_stop_speed -= 0);

				// Decrement running_steps
				master_running_steps--;
			}

			// Ramp down the stepper motor a predetermined amount of steps
			while(master_ramp_down_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -master_pulse_width;
				master_step_counter++;

				////////////////////////////////////////////////////////////
				// This variable is only for testing purposes and it can be
				// deleted, after testing is completed.
				master_steps_made++;
				////////////////////////////////////////////////////////////

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(master_counter += master_start_stop_speed
					+= master_ramp_inc_dec -= 0);

				// Decrement ramp_down_steps
				master_ramp_down_steps--;
			}

			master_move_complete = true;
		}
	}
	while(master_cog > 0);
}